/**
 * @version 3.0.5
 * @Author lu-ch
 * @Email webwork.s@qq.com
 * 文档: https://www.quanzhan.co/luch-request/
 * github: https://github.com/lei-mu/luch-request
 * DCloud: http://ext.dcloud.net.cn/plugin?id=392
 * HBuilderX: beat-3.0.4 alpha-3.0.4
 */

import Request from '@/utils/luch-request/index.js'
import { omit } from 'lodash'
import router from '@/router';

const baseUrl = 'https://api.devops.thinker.vc'

const codeMessage = {
  200: '服务器成功返回请求的数据。',
  201: '新建或修改数据成功。',
  202: '一个请求已经进入后台排队（异步任务）。',
  204: '删除数据成功。',
  400: '发出的请求有错误，服务器没有进行新建或修改数据的操作。',
  401: '用户没有权限（令牌、用户名、密码错误）。',
  403: '用户得到授权，但是访问是被禁止的。',
  404: '发出的请求针对的是不存在的记录，服务器没有进行操作。',
  405: '请求方法不被允许。',
  406: '请求的格式不可得。',
  410: '请求的资源被永久删除，且不会再得到的。',
  422: '当创建一个对象时，发生一个验证错误。',
  500: '服务器发生错误，请检查服务器。',
  502: '网关错误。',
  503: '服务不可用，服务器暂时过载或维护。',
  504: '网关超时。',
};

const getTokenStorage = () => {
	let token = ''
	try {
		token = uni.getStorageSync('Authorization')
	} catch (e) {}
	return token
}

const handleError = (response) => {
	if (response && response.statusCode) {
	    const errorText = response?.data?.message || codeMessage[response.statusCode];
	    const { statusCode } = response;
	    if (statusCode === 401) {
	      uni.reLaunch({
	      	url: '/pages/login/index'
	      });
	      return false;
	    }
	    uni.showToast({
	    	title: statusCode === 401 ? '登录过期，请重新登录！' : errorText,
			icon:'none'
	    })
	  }
	  if (!response) {
	    uni.showToast({
	      title: '您的网络发生异常，无法连接服务器',
		  icon:'none'
	    });
	  }
}

const http = new Request()
http.setConfig((config) => {
	/* 设置全局配置 */
	config.baseURL = baseUrl /* 根域名不同 */
	config.header = {
		...config.header,
		// a: 1, // 演示
		// b: 2 // 演示
	}
	return config
})


http.interceptors.request.use((config) => {
	/* 请求之前拦截器。可以使用async await 做异步操作 */
	const token = getTokenStorage() || config.header.Authorization;
	const inputToken = config.header.Authorization
	config.header = {
		...config.header,
		Authorization: `bearer ${token}`
	}
	if (config?.url?.indexOf('/authorization-server/oauth/token') > -1) {
		config.header.Authorization = inputToken;
	}
	console.log(config, `++++++++++++`)

	if (!config.header.Authorization) { // 如果token不存在，return Promise.reject(config) 会取消本次请求
		return Promise.reject(config)
	}
	return config
}, (config) => {
	return Promise.reject(config)
})


http.interceptors.response.use(async (response) => {
	console.log(response, `PPPPPPPPPPPPPPPPP`)
	/* 请求之后拦截器。可以使用async await 做异步操作  */
	if (response.statusCode !== 200 && response.code !== 200) { // 服务端返回的状态码不等于200，则reject()
	  return Promise.reject(response)
	}
	return response
}, async (response) => { // 请求错误做点什么。可以使用async await 做异步操作
	console.log(response, `::::::::::::::::::::`)
	await handleError(response)
	return Promise.reject(response)
})

export {
	http,
}
